
!* --------------------------------------------------------------------- HICUT
      subroutine hicut(s,nd,fcut,delt,n,icaus)

!  Butterworth hicut filter order 2*n (n<=8) (see Kanasewich, 
!    Time Sequence Analysis in Geophysics, Third Edition, 
!    University of Alberta Press,1981)
!  written by W. B. Joyner 01/07/97
!  causal if icaus.eq.1 - zero phase shift otherwise
!  s(j) input = the time series to be filtered - output = the 
!    filtered series
!  dimension of s(j) must be at least as large as
!    nd+3.0*float(n)/(fcut*delt)
!  nd = the number of points in the time series
!  fcut = the cutoff frequency
!  delt = the timestep
! Dates: xx/xx/xx - Written by Bill Joyner
!        01/07/00 - D. Boore added check for fcut = 0.0d0, in which case
!                   no filter is applied.  He also cleaned up the
!                   appearance of the code (indented statements in
!                   loops, etc.)
!        09/12/00 - double precision statements added so that the output
!                   series has sufficient precision for double integration.
!        11/08/00 - Increased dimension of s from 50000 to 100000
!        02/14/01 - Modified/corrected real numbers function calls to 
!                   double precision - cds
!        02/22/01 - Removed dimension of s (it is up to the user to specify
!                   it properly)
!        07/20/01 - No filter of fcut .le. 0.0
!        11/18/12 - Increased array dimensions for that n<=16

      real s(*)
      real*8 fact(16),b1(16),b2(16)
!      real*8 fact(8),b1(8),b2(8)
      real*8 pi,w1,xp,yp,x1,x2,y1,y2

      if (fcut <= 0.0) return       ! Added by DMB

      pi=4.0d0*datan(1.0d0)
      w1=2.0d0*pi*fcut
      w1=2.0d0*dtan(w1*delt/2.0d0)/delt

      do k = 1, n
        fact(k)=1.0d0/((2.0d0/(w1*delt))**2+
     :   4.0d0*dsin(pi*dfloat(2*k-1)/dfloat(4*n))/(w1*delt)+1.0d0)
        b1(k)=2.0d0*(1.0d0-(2.0d0/(w1*delt))**2)*fact(k)
        b2(k)=((2.0d0/(w1*delt))**2-
     :   4.0d0*dsin(pi*dfloat(2*k-1)/dfloat(4*n))/(w1*delt)+1.0d0)
     :   *fact(k)
      end do

      np2=nd

      if (icaus /= 1) then
        npad=3.0d0*dfloat(n)/(fcut*delt)
        np1=nd+1
        np2=nd+npad
        do j = np1, np2
          s(j)=0.0
        end do
      end if

      do k = 1, n
        x1=0.0d0
        x2=0.0d0
        y1=0.0d0
        y2=0.0d0
        do j = 1, np2
          xp=s(j)
          yp=fact(k)*(xp+2.0d0*x1+x2)-b1(k)*y1-b2(k)*y2
          s(j)=yp
          y2=y1
          y1=yp
          x2=x1
          x1=xp
        end do
      end do

      if(icaus /= 1) then
        do k = 1, n
          x1=0.0d0
          x2=0.0d0
          y1=0.0d0
          y2=0.0d0
          do j = 1, np2
            xp=s(np2-j+1)
            yp=fact(k)*(xp+2.0d0*x1+x2)-b1(k)*y1-b2(k)*y2
            s(np2-j+1)=yp
            y2=y1
            y1=yp
            x2=x1
            x1=xp
          end do
        end do
      end if

      return
      end
!* --------------------------------------------------------------------- HICUT

