      subroutine cav_compute(a, amin4cav, iunits, npts, sps, cav)

!       where iunits is:
!         0 = g
!         1 = cm/s/s
!         2 = m/s/s

! NOTE: This version does not deal with noninteger sps.
     
! Dates: 03/12/10 - Modified by Chris Stephens
!        08/26/10 - Accepted Chris's changes (except add "abs" before array a), 
!                   changed indexing
!                   for each 1-sec segment, treat case of sign change
!                   for adjacent points properly (using
!                   Bozorgnia code \cav\CAV_YB8_OK.for)
!        06/27/19 - Add amin4cav (in g) as an input parameter, so can compute
!                   Standardized CAV (amin4cav=0.025g) or standard CAV (amin4cav=0.0g).  See
!                   email from Ken Campbell regarding differences in our values,
!                   which turned out to be due to my computing Standardized CAV and
!                   he computing standard CAV.

      real a(*)
      

!      amin4cav = 0.025 ! in g  ! Reg Guide 1.166

! this section added by c. stephens

      select case (iunits)
        case (0)
          factor = 1.0		! g
        case (1)
          factor = 981.0	! cm/s/s per g
        case (2)
          factor = 9.81		! m/s/s
        case default
          print *, 'Error (cav_compute): value of iunits invalid'
          stop     ! (might instead return a value for cav < 0. to indicate error in computing value?) 
      end select

! scale threshold from g to input units    
      thresh = amin4cav * factor
      
!DEBUG
      call get_lun(nu_debug_in_sub)
      open(nu_debug_in_sub,file='debug_in_cav_compute.out',
     :     status='unknown')
      write(nu_debug_in_sub,*) 
     :     ' amin4cav, factor, thresh, sps, npts = ', 
     :       amin4cav, factor, thresh, sps, npts
      write(nu_debug_in_sub,'(3x,a, 5x,a, 5x,a, 3x,a)') 
     :     'iseg', 'i1', 'i2', 'iuse'
!DEBUG

      cav = 0.0
      
      iseg = 0
      
!      i2 = 0 ! old
      i2 = 1
      
      loop over 1s segments: DO

!      i1 = i2+ 1 ! old
      i1 = i2
      if (i1 >= npts) then  ! check for the end of the segments
        EXIT loop over 1s segments
      end if
      
      iseg = iseg + 1
      
      i2 = min(npts, i1+int(sps))
      
! Does the acceleration exceed amin in this one-second segment?

      iuse = 0
      do i = i1, i2
        if(abs(a(i)) .gt. thresh) then
          iuse = 1
          exit	! no need to check the rest of the data points
        endif
      end do
      
!DEBUG
      write(nu_debug_in_sub,'(4(1x,i6))' ) iseg, i1, i2, iuse
!DEBUG


! If so, accumulate the absolute acceleration
      if (iuse .eq. 1) then
        sum = 0.0
        do i = i1, i2-1
          if (a(i)*a(i+1) >= 0.0) then      ! no zero crossing
            sum = sum + 0.5*abs(a(i)+a(i+1))
          else                              ! zero crossing
            zc = -a(i)/(a(i+1)-a(i))    ! factor out 1/sps, as it is applied later
            area_1 = abs(a(i))*zc             ! factor out 0.5 factor, applied it later
            area_2 = abs(a(i+1))*(1.0-zc)
            sum = sum + 0.5 * (area_1+area_2)
          end if
        end do

        sum = sum/factor   ! convert to g
        cav = cav + sum/sps
      end if
      
! Go back for another one-second segment
      END DO loop over 1s segments
      
      return
        
      end
      
