!-----------------------------------------------------------------------
      subroutine icmpmx(kug, ug, dt_in, pr, w, d, z)
      
! z(1) = SD
! z(2) = RV
! z(3) = AA

! Dates: 03/06/10 - Original program ucmpmx (written by Bob Youngs/I. Idriss)
!                   renamed icmpmx and modified to assume equal time spacing
!                   if the original and interpolated acceleration time
!                   series (the "u" in the original name referred to 
!                   "u"nequal spacing).
!        05/16/15 - Use Implicit None and replace real*4 and double precision with modern equivalents

      implicit none

! Input
      integer :: kug
      real(8) :: ug(*), pr, w, d, dt_in

! Output
      real(8) :: z(*)

! Working variables
      real(8) :: t(3), c(3), x(2,3)
      real(8) :: f1, f2, f3, f4, f5, f6, wd, w2, w3
      real(8) :: dt, e, g1, g2, h1, h2, dug, g, z1, z2, z3, z4
      real(8) :: a, b
      integer :: nn, i, k, ns, is, j
!
      nn=1
      wd=sqrt(1.-d*d)*w
      w2=w*w
      w3=w2*w
      DO i=1,3
        x(1,i)=0.
        z(i)=0.
      END DO
      
      f2=1./w2
      f3=d*w
      f4=1./wd
      f5=f3*f4
      f6=2.*f3
      
      ns= int(10.*dt_in/pr-0.01)+1   !! 05/05/2008
      dt=dt_in/real(ns)
      
      DO k=1,kug
      
        f1=2.*d/w3/dt
        e=dexp(-f3*dt)
        g1=e*dsin(wd*dt)
        g2=e*dcos(wd*dt)
        h1=wd*g2-f3*g1
        h2=wd*g1+f3*g2
        dug=(ug(k+1)-ug(k))/real(ns)
        g=ug(k)
        z1=f2*dug
        z3=f1*dug
        z4=z1/dt
        
        DO is=1,ns
          z2=f2*g
          b=x(1,1)+z2-z3
          a=f4*x(1,2)+f5*b+f4*z4
          x(2,1)=a*g1+b*g2+z3-z2-z1
          x(2,2)=a*h1-b*h2-z4
          x(2,3)=-f6*x(2,2)-w2*x(2,1)
          nn = nn + 1
          DO j=1,3
            c(j)=abs(x(2,j))
            IF (c(j) > z(j)) THEN
              z(j)=c(j)
            END IF
            x(1,j)=x(2,j)
          END DO
          
          g=g+dug
          
        END DO
      END DO
  
      RETURN
      END

!------------------------------
 
